import os
import glob
from collections import OrderedDict
from . import config


POSTRENDER_REGISTRY = OrderedDict()


def register_postrender(name, fn, default=None):
    '''Add postrender function to registry

    :param name: Name of postrender function (used as label in ui)
    :param fn: Postrender function
    '''

    fn.default = default
    POSTRENDER_REGISTRY[name] = fn


def unregister_postrender(name):
    '''Remove postrender function from registry'''

    POSTRENDER_REGISTRY.pop(name, None)


PATHGEN_REGISTRY = OrderedDict()


def register_path(name, fn):
    '''Add a path generator function to registry

    :param name: Name of path generator (used as label in ui)
    :param fn: Path generator function
    '''

    PATHGEN_REGISTRY[name] = fn


def unregister_path(name):
    '''Remove path generator function from registry

    :param name: Name of path generator function
    '''

    PATHGEN_REGISTRY.pop(name, None)


def init():
    '''Import all python files in config.PRESETS_PATH'''

    for f in glob.glob(os.path.join(config.PRESETS_PATH, '*.py')):

        base = os.path.basename(f)
        name = os.path.splitext(base)[0]
        __import__(name)



import json
import glob
import os
from . import config


def get_presets():
    '''Get a generator yielding preset name, data pairs'''

    for f in glob.glob(os.path.join(config.PRESETS_PATH, '*.json')):

        base = os.path.basename(f)
        name = os.path.splitext(base)[0]

        with open(f, 'r') as f:
            data = json.loads(f.read())

        yield name, data


def get_preset(name):
    '''Get a preset by name'''

    for n, s in get_presets():
        if name == n:
            return s


def new_preset(name, data):
    '''Create a new preset from viewport state data

    :param name: Name of the preset
    :param data: Viewport state dict

    usage::

        import mvp
        active = mvp.Viewport.active()
        mvp.new_preset('NewPreset1', active.get_state())
    '''

    preset_path = os.path.join(config.PRESETS_PATH, name + '.json')
    with open(preset_path, 'w') as f:
        f.write(json.dumps(data))


def del_preset(name):

    preset_path = os.path.join(config.PRESETS_PATH, name + '.json')
    if os.path.exists(preset_path):
        os.remove(preset_path)



'''
Configure mvp presets and hooks
'''

import os
import sys

PRESETS_PATH = os.environ.get('MVP_PRESETS', os.path.expanduser('~/.mvp'))

if not os.path.exists(PRESETS_PATH):
    os.makedirs(PRESETS_PATH)

sys.path.insert(1, PRESETS_PATH)



from Qt import QtWidgets
import time


def get_maya_window(cache=[]):
    '''Get Maya MainWindow as a QWidget.'''

    if cache:
        return cache[0]

    for widget in QtWidgets.QApplication.instance().topLevelWidgets():
        if widget.objectName() == 'MayaWindow':
            cache.append(widget)
            return widget

    raise RuntimeError('Could not locate MayaWindow...')


def wait(delay=1):
    '''Delay python execution for a specified amount of time'''

    app = QtWidgets.QApplication.instance()

    s = time.clock()
    while True:
        if time.clock() - s >= delay:
            return

        app.processEvents()
